<?php

namespace App\Providers;

use App\Template;
use Illuminate\Foundation\Support\Providers\RouteServiceProvider as ServiceProvider;
use Illuminate\Support\Facades\Route;

class RouteServiceProvider extends ServiceProvider
{
    /**
     * This namespace is applied to your controller routes.
     *
     * In addition, it is set as the URL generator's root namespace.
     *
     * @var string
     */
    protected $namespace = 'App\Http\Controllers';

    /**
     * The path to the "home" route for your application.
     *
     * @var string
     */
    public const HOME = '/home';

    /**
     * Define your route model bindings, pattern filters, etc.
     *
     * @return void
     */
    public function boot()
    {
        //

        parent::boot();
    }

    /**
     * Define the routes for the application.
     *
     * @return void
     */
    public function map()
    {
        $this->mapApiRoutes();

        $this->mapWebRoutes();

        //
    }

    /**
     * Define the "web" routes for the application.
     *
     * These routes all receive session state, CSRF protection, etc.
     *
     * @return void
     */
    protected function mapWebRoutes()
    {
        Route::middleware('web')
            ->namespace($this->namespace)
            ->group(base_path('routes/web.php'));
        if(defined('TLAYOUT')){
            $currentTemplate = Template::where('enabled',1)->first();
            if($currentTemplate){
                  $routeFile = TEMPLATE_PATH.'/'.$currentTemplate->directory.'/routes.php';
                if(file_exists($routeFile)){
                    Route::middleware('web')
                        ->namespace($this->namespace)
                        ->group(public_path(TEMPLATE_PATH.'/'.$currentTemplate->directory.'/routes.php'));
                }

            }


        }
    }

    /**
     * Define the "api" routes for the application.
     *
     * These routes are typically stateless.
     *
     * @return void
     */
    protected function mapApiRoutes()
    {
        Route::prefix('api')
            ->middleware(['api','api-base'])
            ->namespace($this->namespace)
            ->group(base_path('routes/api.php'));
    }
}
